function [Ypca] = PCA(X,r)
%
% Syntax:       Ypca = PCA(X,r);
%               
% Inputs:       X is an d x n matrix containing n samples of d-dimensional
%               data
%               
%               r is the number of principal components to compute
%               
% Outputs:      Ypca is an r x n matrix containing the r principal
%               components - scaled to variance 1 - of the input samples
%               
%               
% Description:  Performs principal component analysis (PCA) on the input
%               data
%               
%

% Center data
[Xc, ~] = centerRows(X);

% Compute truncated SVD
%[U, S, V] = svds(Zc,r); % Equivalent, but usually slower than svd()
[U, S, V] = svd(Xc,'econ');
U = U(:,1:r);
S = S(1:r,1:r);
V = V(:,1:r);

% Compute principal components
Ypca = S * V';
%Ypca = U' * Xc; % Equivalent but slower

