function w = hammingf(L,sflag)
%HAMMINGF Reimplementation of Hamming function from Signal Processing
%Toolbox
%   w = hammingf(L) returns an L-point symmetric Hamming window
%   w = hamming(L,sflag) returns a Hamming window using the window sampling
%   specified by _sflag_.
%
%   Input arguments:
%   L - Window length (positive integer)
%   sflag - 'symmetric' (default) | 'periodic'
%       'symmetric'  Use this option when using windows for filter design.
%       'periodic'  This option is useful for spectral analysis because
%           it enables a windowed signal to have the perfect periodic
%           extension implicit in the discrete Fourier transform. When
%           'periodic' is specified, hamming computes a window of length
%           L + 1 and returns the first L points.

if ~exist('sflag','var')
    sflag = 'symmetric';
end

switch sflag
    case 'symmetric'
        ind = (0:L-1)';
    case 'periodic'
        ind = (0:L)';
    otherwise
        ME = MException('Invalid argument', ...
            'Flag argument does not match one of the possible options');
        throw(ME);
end
w = 0.54 - 0.46 * cos(2*pi*(ind/(L-1)));

if strcmp(sflag,'periodic')
    w = w(1:end-1);
end

end

