%% Blind Source Separation, Skript 2
% In diesem Skript wird der gesamte Prozess der Blind Source Separation durchlaufen, 
% dazu werden zwei echte Aufnahmen als Input in den Algorithmus eingeladen. Die 
% ICA findet im Frequenzbereich der Signale statt, dazu werden STFT und ISTFT 
% zur Umwandlung verwendet.
%% Initialisierung
%%
clear all
close all
clc
run setupPath.m
%% Laden Sie die vorgefertigten Sprachaufnahmen ein mittels getMaleSpeech und getFemaleSpeech (./contributed/util)
%%
samplerate = 16000; % Hz = Abtastungen pro Sekunde

s1 = getFemaleSpeech(3); 
s2 = getMaleSpeech(3);

% Stellen Sie sicher, dass die Signallngen gleich sind
% (Abschneiden auf die kleinere Lnge)
signalLength = min([length(s1),length(s2)]);
s1 = s1(1:signalLength);
s2 = s2(1:signalLength);

% Normalisieren Sie die Amplituden
s1 = s1 / max(abs(s1));
s2 = s2 / max(abs(s2));

% Berechnen Sie einen Zeitvektor basierend auf der ...
%   Signallnge und der Abtastrate
timevector_s = 0 : 1/samplerate : (signalLength-1)/samplerate;

%% Initialisierungen fr den RIR-Generator und STFT (vorgegeben)
%%
%% Initialisierung
soundVelocity =   340;       % m/s
rirLength     =  2048;       % Abtastungen

% room layout
reverberationTime = 0.13;           % in Sek
roomDimension     = [7 5 3];        % [x y z] m
receiverPositions = [2 3 1; 2 1 1]; % [x y z] m
sourcePositions   = [5 3 1; 5 1 1]; % [x y z] m

% stft configuration
windowType   = 'hamming';    % aus {'hamming', 'hann', 'gauss', 'tukey'}
windowLength = 4096;         % Abtastungen
windowHop    = 1024;         % Abtastungen
fftLength    = 4096;         % Abtastungen

%% Generieren Sie eine Mischmatrix unter Verwendung von rir_generator (./contributed/rir_generator)
%%
% Es ist mglich, dass Sie an dieser Stelle einen C++-Compiler
% installieren mssen, aber versuchen Sie zunchst: mex -setup
% Wenn Sie bereits einen Compiler haben, whlen Sie ihn aus: mex -setup C++
% Erstellen Sie eine .mex-Datei: mex rir_generator.cpp
% Erstellen Sie die Raumimpulsantworten

rir = zeros(size(receiverPositions, 1), size(sourcePositions, 1), rirLength);
for count_receiver = 1 : size(receiverPositions, 1) % fr jeden Empfnger
    for count_source = 1 : size(sourcePositions, 1) % fr jede Quelle
        % erstellen Sie Impulse der Lnge rirLength fr jedes Paar
        rir(count_receiver,count_source,:) = rir_generator(soundVelocity, samplerate, ...
            receiverPositions(count_receiver,:), sourcePositions(count_source,:), roomDimension, ...
            reverberationTime, rirLength);
    end
end
%% Initialisierungen fr den RIR-Generator und STFT (vorgegeben)
%%
RIR = fft(rir, fftLength, 3);
RIR = RIR(:, :, 1:ceil((1+fftLength)/2));
stft = STFT(windowLength, windowHop, windowType, fftLength, samplerate);
%% Generieren Sie die Mischsignale durch Faltung von RIR und Quellsignalen; Speichern Sie diese in den Matrizen x1 und x2
%%
x1 = conv(squeeze(rir(1,1,:)), s1) + conv(squeeze(rir(1,2,:)), s2);
x2 = conv(squeeze(rir(2,1,:)), s1) + conv(squeeze(rir(2,2,:)), s2);
% Berechnen Sie einen Zeitvektor basierend auf der ...
%   Signallnge von x1/x2 und der Abtastrate zum Plotten
timevector_x = 0 : 1/samplerate : (length(x1)-1)/samplerate;
%% Wenden Sie STFT auf die Mischsignale an
%%
[X1, timeslots, freqbins] = stft.stft(x1);
X2 = stft.stft(x2);

numFreqbins = length(freqbins);
numTimeslots = length(timeslots);
%% Wenden Sie die ICA an und erhalten Sie A und W
%%
Y1 = zeros(size(X1)); % Initialisierung fr das wiederhergestellte Signal 1
Y2 = zeros(size(X2)); % Initialisierung fr das wiederhergestellte Signal 2
W = zeros(2, 2, numFreqbins); % Initialisierung Wiederherstellungsmatrix
A = zeros(2, 2, numFreqbins); % Initialisierung Mischmatrix

for count_freqbin = 1 : numFreqbins % fr jeden Frequenzbin wenden Sie ICA an
    [Ytemp, Q, T, ~] = ...
        fastICA([X1(count_freqbin,:); X2(count_freqbin,:)], 2, 'kurtosis', 0);
    A(:,:,count_freqbin) = inv(Q*T); % Erhalten Sie die Mischmatrix
    W(:,:,count_freqbin) = Q*T; % Erhalten Sie die Wiederherstellungsmatrix
    Y1(count_freqbin,:) = Ytemp(1,:); % Erhalten Sie das wiederhergestellte Signal
    Y2(count_freqbin,:) = Ytemp(2,:); % Erhalten Sie das wiederhergestellte Signal
end

%% Permutationsausrichtung und Skalierungskorrektur
%%
perm = permutationAlignmentDyadic(A, Y1, Y2, samplerate, 1);

for count_freqbin = 1 : numFreqbins  % fr jeden Frequenzbin fhren Sie PA und SK aus
    
    % Anwendung von PA auf jedes Frequenz-Bin in W
    W(:,:,count_freqbin) = perm(:,:,count_freqbin)*W(:,:,count_freqbin);
    
    % Anwendung von SK auf jedes Frequenz-Bin in W
    W(:,:,count_freqbin) = diag(diag(inv(W(:,:,count_freqbin))))*W(:,:,count_freqbin);
    
    % Trennung
    Ytemp = W(:,:,count_freqbin)*centerRows([X1(count_freqbin,:); X2(count_freqbin,:)]);
    Y1(count_freqbin,:) = Ytemp(1,:);
    Y2(count_freqbin,:) = Ytemp(2,:);
    

end
clear freqbin
%% Wenden Sie ISTFT an, um Signale in den Zeitbereich zu transformieren
%%
[y1, timevector_y] = stft.istft(Y1);
y2 = stft.istft(Y2);
%% Evaluieren Sie die Performance durch Plotten der Quellsignale, Mischsignale und wiederhergestellten Signale
%%
fig = figure;
resizeFigure(fig, 800, 600);

subplot(3,2,1);
plot(timevector_s, s1, 'k');
title('Original Signal 1');
xlabel('Time (s)');
ylabel('Amplitude');
subplot(3,2,2);
plot(timevector_s, s2, 'k');
title('Original Signal 2');
xlabel('Time (s)');
ylabel('Amplitude');

subplot(3,2,3);
plot(timevector_x, x1, 'k');
ylim([-0.04,0.04]);
title('Mixed Signal 1');
xlabel('Time (s)');
ylabel('Amplitude');
subplot(3,2,4);
plot(timevector_x, x2, 'k');
ylim([-0.04,0.04]);
title('Mixed Signal 2');
xlabel('Time (s)');
ylabel('Amplitude');

subplot(3,2,5);
plot(timevector_y, y1, 'k');
ylim([-0.02,0.02]);
title('Recovered Signal 1');
xlabel('Time (s)');
ylabel('Amplitude');
subplot(3,2,6);
plot(timevector_y, y2, 'k');
ylim([-0.02,0.02]);
title('Recovered Signal 2');
xlabel('Time (s)');
ylabel('Amplitude');
minLength = min([length(s1),length(y1)]);
s1c = s1(1:minLength);
s2c = s2(1:minLength);
y1c = y1(1:minLength);
y2c = y2(1:minLength);
%% Hren Sie sich die Signale an mit der Funktion sound(signal, sample_rate_of_signal)
%% Evaluation durch Korrelationskoeffizienten (Extra)
%%
minLength = min([length(s1),length(y1)]);
s1c = s1(1:minLength);
s2c = s2(1:minLength);
y1c = y1(1:minLength);
y2c = y2(1:minLength);
[corrMatrix, shiftMatrix] = corrCoeffShift([s1c;s2c], [y1c;y2c]);
[mainDiag, secondDiag] = getAvgDiagValues(corrMatrix);
clear minLength;

disp('Correlation Matrix:');
disp(corrMatrix);